@@ -218,7 +218,7 @@ get_remote_ssh_command (const char *uri,
 
 	quoted_remote_command = g_shell_quote (remote_command);
 
-	command = g_strdup_printf ("ssh %s -t %s", user_host, quoted_remote_command);
+	command = g_strdup_printf ("ssh %s%s -t %s", user_host, port_str, quoted_remote_command);
 
 	g_free (user_name);
 	g_free (user_host);
@@ -504,13 +504,19 @@ nautilus_open_terminal_get_background_items (NautilusMenuProvider *provider,
 	uri = nautilus_file_info_get_activation_uri (file_info);
 	terminal_file_info = get_terminal_file_info (uri);
 
-	if (terminal_file_info == FILE_INFO_SFTP || uri_has_local_path (uri)) {
+	if (terminal_file_info == FILE_INFO_SFTP ||
+	    terminal_file_info == FILE_INFO_DESKTOP ||
+	    uri_has_local_path (uri)) {
+		/* local locations or SSH */
 		item = open_terminal_menu_item_new (file_info, terminal_file_info, gtk_widget_get_screen (window),
 						    NULL, terminal_file_info == FILE_INFO_SFTP, FALSE);
 		items = g_list_append (items, item);
 	}
 
-	if (terminal_file_info == FILE_INFO_SFTP && uri_has_local_path (uri)) {
+	if ((terminal_file_info == FILE_INFO_SFTP ||
+	     terminal_file_info == FILE_INFO_OTHER) &&
+	    uri_has_local_path (uri)) {
+		/* remote locations that offer local back-mapping */
 		item = open_terminal_menu_item_new (file_info, terminal_file_info, gtk_widget_get_screen (window),
 						    NULL, FALSE, FALSE);
 		items = g_list_append (items, item);
@@ -518,7 +524,9 @@ nautilus_open_terminal_get_background_items (NautilusMenuProvider *provider,
 
 	if (display_mc_item () &&
 	    g_find_program_in_path ("mc") &&
-	    uri_has_local_path (uri)) {
+	    ((terminal_file_info == FILE_INFO_DESKTOP &&
+	      (desktop_is_home_dir () || desktop_opens_home_dir ())) ||
+	     uri_has_local_path (uri))) {
 		item = open_terminal_menu_item_new (file_info, terminal_file_info, gtk_widget_get_screen (window), "mc", FALSE, FALSE);
 		items = g_list_append (items, item);
 	}
